@@ -1,7 +1,7 @@
 /*
  *  WEP Cloaking filtering
  *
- *  Copyright (C) 2008-2020 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
+ *  Copyright (C) 2008-2022 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
  *
  *  Thanks to Alex Hernandez aka alt3kx for the hardware.
  *
@@ -53,7 +53,9 @@ static unsigned char buffer[65536];
 
 static char * _filename_output_invalid;
 static char * _filename_output_cloaked;
+static size_t _filename_output_cloaked_len;
 static char * _filename_output_filtered;
+static size_t _filename_output_filtered_len;
 static FILE * _output_cloaked_packets_file;
 static FILE * _output_clean_capture_file;
 static FILE * _input_file;
@@ -320,13 +322,11 @@ static int get_rtap_signal(int caplen)
 // !!!! WDS not yet implemented
 static BOOLEAN read_packets(void)
 {
-	int i, start;
+	int start;
 	time_t tt;
 	unsigned char * h80211;
 	size_t bytes_read;
 
-	i = 0;
-
 	memset(&stats, 0, sizeof(stats));
 	tt = time(NULL);
 
@@ -484,7 +484,8 @@ static BOOLEAN read_packets(void)
 			// future)
 			_packet_elt_head->current->is_cloaked = VALID_FRAME_UNCLOAKED;
 		}
-		else if (_packet_elt_head->current->frame_type == FRAME_TYPE_DATA)
+		else if (_packet_elt_head->current->frame_type //-V547
+				 == FRAME_TYPE_DATA)
 		{
 			_packet_elt_head->current->is_cloaked
 				= UKNOWN_FRAME_CLOAKING_STATUS;
@@ -746,14 +747,7 @@ static BOOLEAN next_packet_pointer_from_ap(void)
 			return false;
 		}
 	}
-	if (_packet_elt_head->current->toDS == 0)
-	{
-		return true;
-	}
-	else
-	{
-		return false;
-	}
+	return true;
 }
 
 static BOOLEAN next_packet_pointer_from_client(void)
@@ -1089,9 +1083,7 @@ static int CFC_filter_signal(void)
 						++nb_packets;
 						break;
 					} //-V796
-#if __GNUC__ >= 7
-					__attribute__((fallthrough));
-#endif
+					fallthrough;
 				case UKNOWN_FRAME_CLOAKING_STATUS:
 					// If variation is > max allowed variation, it's a cloaked
 					// packet
@@ -1247,9 +1239,7 @@ static int CFC_filter_duplicate_iv(void)
 						_packet_elt_head->current->is_cloaked = CLOAKED_FRAME;
 						++nb_packets;
 					} //-V796
-#if __GNUC__ >= 7
-					__attribute__((fallthrough));
-#endif
+					fallthrough;
 				case UKNOWN_FRAME_CLOAKING_STATUS:
 					// If unknown status, mark it as potentially cloaked
 					if (*(ivs_table + get_iv(_packet_elt_head->current)) > 1)
@@ -1502,7 +1492,7 @@ static void usage(void)
 		"Airdecloak-ng", _MAJ, _MIN, _SUB_MIN, _REVISION, _BETA, _RC);
 	printf(
 		"\n"
-		"  %s - (C) 2008-2020 Thomas d\'Otreppe\n"
+		"  %s - (C) 2008-2022 Thomas d\'Otreppe\n"
 		"  https://www.aircrack-ng.org\n"
 		"\n"
 		"  usage: airdecloak-ng [options]\n"
@@ -1562,6 +1552,7 @@ int main(int argc, char * argv[])
 {
 	int temp = 0, option;
 	int manual_cloaked_fname = 0, manual_filtered_fname = 0;
+	//-V:tempBool:1048
 	BOOLEAN tempBool;
 	char * input_filename;
 	char * input_bssid;
@@ -1614,12 +1605,7 @@ int main(int argc, char * argv[])
 		switch (option)
 		{
 			case ':':
-
-				printf("\"%s --help\" for help.\n", argv[0]);
-				return (EXIT_FAILURE);
-
 			case '?':
-
 				printf("\"%s --help\" for help.\n", argv[0]);
 				return (EXIT_FAILURE);
 			case 'a':
@@ -1632,6 +1618,7 @@ int main(int argc, char * argv[])
 				if (optarg != NULL)
 				{
 					_filename_output_cloaked = optarg;
+					_filename_output_cloaked_len = strlen(optarg) + 16;
 					manual_cloaked_fname = 1;
 				}
 				break;
@@ -1639,6 +1626,7 @@ int main(int argc, char * argv[])
 				if (optarg != NULL)
 				{
 					_filename_output_filtered = optarg;
+					_filename_output_filtered_len = strlen(optarg) + 16;
 					manual_filtered_fname = 1;
 				}
 				break;
@@ -1790,13 +1778,17 @@ int main(int argc, char * argv[])
 		temp = strlen(input_filename);
 		if (!manual_cloaked_fname)
 		{
-			_filename_output_cloaked = (char *) calloc(temp + 9 + 5, 1);
+			_filename_output_cloaked_len = temp + 9 + 5;
+			_filename_output_cloaked
+				= (char *) calloc(_filename_output_cloaked_len, 1);
 			ALLEGE(_filename_output_cloaked != NULL);
 		}
 
 		if (!manual_filtered_fname)
 		{
-			_filename_output_filtered = (char *) calloc(temp + 10 + 5, 1);
+			_filename_output_filtered_len = temp + 10 + 5;
+			_filename_output_filtered
+				= (char *) calloc(_filename_output_filtered_len, 1);
 			ALLEGE(_filename_output_filtered != NULL);
 		}
 
@@ -1810,12 +1802,12 @@ int main(int argc, char * argv[])
 		{
 			if (!manual_cloaked_fname)
 				snprintf(_filename_output_cloaked,
-						 strlen(input_filename) + 9 + 5,
+						 _filename_output_cloaked_len,
 						 "%s-cloaked.pcap",
 						 input_filename);
 			if (!manual_filtered_fname)
 				snprintf(_filename_output_filtered,
-						 strlen(input_filename) + 10 + 5,
+						 _filename_output_filtered_len,
 						 "%s-filtered.pcap",
 						 input_filename);
 		}
@@ -1823,17 +1815,21 @@ int main(int argc, char * argv[])
 		{
 			if (!manual_cloaked_fname)
 			{
-				strncpy(_filename_output_cloaked,
+				strlcpy(_filename_output_cloaked,
 						input_filename,
-						strlen(input_filename) + 9 + 5 - 1);
-				strncat(_filename_output_cloaked, "-cloaked.pcap", 14);
+						_filename_output_cloaked_len);
+				strlcat(_filename_output_cloaked,
+						"-cloaked.pcap",
+						_filename_output_cloaked_len);
 			}
 			if (!manual_filtered_fname)
 			{
-				strncpy(_filename_output_filtered,
+				strlcpy(_filename_output_filtered,
 						input_filename,
-						strlen(input_filename) + 10 + 5 - 1);
-				strncat(_filename_output_filtered, "-filtered.pcap", 15);
+						_filename_output_filtered_len);
+				strlcat(_filename_output_filtered,
+						"-filtered.pcap",
+						_filename_output_filtered_len);
 			}
 		}
 	}
