@@ -1,7 +1,7 @@
 /*
  *  Airodump-ng text files output
  *
- *  Copyright (C) 2018-2020 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
+ *  Copyright (C) 2018-2022 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -180,6 +180,7 @@ int dump_write_csv(struct AP_info * ap_1st,
 				ap_cur->bssid[5]);
 
 		ltime = localtime(&ap_cur->tinit);
+		REQUIRE(ltime != NULL);
 
 		fprintf(opt.f_txt,
 				"%04d-%02d-%02d %02d:%02d:%02d, ",
@@ -191,6 +192,7 @@ int dump_write_csv(struct AP_info * ap_1st,
 				ltime->tm_sec);
 
 		ltime = localtime(&ap_cur->tlast);
+		REQUIRE(ltime != NULL);
 
 		fprintf(opt.f_txt,
 				"%04d-%02d-%02d %02d:%02d:%02d, ",
@@ -203,7 +205,9 @@ int dump_write_csv(struct AP_info * ap_1st,
 
 		fprintf(opt.f_txt, "%2d, %3d,", ap_cur->channel, ap_cur->max_speed);
 
-		if ((ap_cur->security & (STD_OPN | STD_WEP | STD_WPA | STD_WPA2)) == 0)
+		if ((ap_cur->security
+			 & (STD_OPN | STD_WEP | STD_WPA | STD_WPA2 | AUTH_SAE | AUTH_OWE))
+			== 0)
 			fprintf(opt.f_txt, " ");
 		else
 		{
@@ -284,11 +288,12 @@ int dump_write_csv(struct AP_info * ap_1st,
 
 		if (ap_cur->key != NULL)
 		{
-			for (i = 0; i < (int) strlen(ap_cur->key); i++)
+			const int key_len = strlen(ap_cur->key);
+
+			for (i = 0; i < key_len; i++)
 			{
 				fprintf(opt.f_txt, "%02X", ap_cur->key[i]);
-				if (i < (int) (strlen(ap_cur->key) - 1))
-					fprintf(opt.f_txt, ":");
+				if (i < (key_len - 1)) fprintf(opt.f_txt, ":");
 			}
 		}
 
@@ -323,6 +328,7 @@ int dump_write_csv(struct AP_info * ap_1st,
 				st_cur->stmac[5]);
 
 		ltime = localtime(&st_cur->tinit);
+		REQUIRE(ltime != NULL);
 
 		fprintf(opt.f_txt,
 				"%04d-%02d-%02d %02d:%02d:%02d, ",
@@ -334,6 +340,7 @@ int dump_write_csv(struct AP_info * ap_1st,
 				ltime->tm_sec);
 
 		ltime = localtime(&st_cur->tlast);
+		REQUIRE(ltime != NULL);
 
 		fprintf(opt.f_txt,
 				"%04d-%02d-%02d %02d:%02d:%02d, ",
@@ -413,6 +420,7 @@ int dump_write_airodump_ng_logcsv_add_ap(const struct AP_info * ap_cur,
 
 	// Local computer time
 	const struct tm * ltime = localtime(&ap_cur->tlast);
+	REQUIRE(ltime != NULL);
 	fprintf(opt.f_logcsv,
 			"%04d-%02d-%02d %02d:%02d:%02d,",
 			1900 + ltime->tm_year,
@@ -485,6 +493,7 @@ int dump_write_airodump_ng_logcsv_add_client(const struct AP_info * ap_cur,
 
 	// Local computer time
 	struct tm * ltime = localtime(&ap_cur->tlast);
+	REQUIRE(ltime != NULL);
 	fprintf(opt.f_logcsv,
 			"%04d-%02d-%02d %02d:%02d:%02d,",
 			1900 + ltime->tm_year,
@@ -605,7 +614,7 @@ char * get_manufacturer_from_string(char * buffer)
 {
 	char * manuf = NULL;
 	char * buffer_manuf;
-	if (buffer != NULL && strlen(buffer) > 0)
+	if (buffer != NULL && *buffer != '\0')
 	{
 		buffer_manuf = strstr(buffer, "(hex)");
 		if (buffer_manuf != NULL)
@@ -621,17 +630,18 @@ char * get_manufacturer_from_string(char * buffer)
 			// Did we stop at the manufacturer
 			if (*buffer_manuf != '\0')
 			{
+				const int buffer_manuf_len_minus_1 = strlen(buffer_manuf);
 
 				// First make sure there's no end of line
-				if (buffer_manuf[strlen(buffer_manuf) - 1] == '\n'
-					|| buffer_manuf[strlen(buffer_manuf) - 1] == '\r')
+				if (buffer_manuf[buffer_manuf_len_minus_1] == '\n'
+					|| buffer_manuf[buffer_manuf_len_minus_1] == '\r')
 				{
-					buffer_manuf[strlen(buffer_manuf) - 1] = '\0';
-					if (*buffer_manuf != '\0'
-						&& (buffer_manuf[strlen(buffer_manuf) - 1] == '\n'
-							|| buffer[strlen(buffer_manuf) - 1] == '\r'))
+					buffer_manuf[buffer_manuf_len_minus_1] = '\0';
+					if (buffer_manuf_len_minus_1 >= 1
+						&& (buffer_manuf[buffer_manuf_len_minus_1 - 1] == '\n'
+							|| buffer[buffer_manuf_len_minus_1 - 1] == '\r'))
 					{
-						buffer_manuf[strlen(buffer_manuf) - 1] = '\0';
+						buffer_manuf[buffer_manuf_len_minus_1 - 1] = '\0';
 					}
 				}
 				if (*buffer_manuf != '\0')
@@ -682,9 +692,11 @@ static int dump_write_kismet_netxml_client_info(struct ST_info * client,
 					   || memcmp(client->base->bssid, BROADCAST, 6) == 0);
 
 	strncpy(first_time, ctime(&client->tinit), TIME_STR_LENGTH - 1);
+	ENSURE(strlen(first_time) >= 1);
 	first_time[strlen(first_time) - 1] = 0; // remove new line
 
 	strncpy(last_time, ctime(&client->tlast), TIME_STR_LENGTH - 1);
+	ENSURE(strlen(last_time) >= 1);
 	last_time[strlen(last_time) - 1] = 0; // remove new line
 
 	fprintf(opt.f_kis_xml,
@@ -765,12 +777,7 @@ static int dump_write_kismet_netxml_client_info(struct ST_info * client,
 															: client->rate_to;
 	fprintf(opt.f_kis_xml,
 			"\t\t\t<maxseenrate>%.6f</maxseenrate>\n",
-			client_max_rate /
-#if defined(__x86_64__) && defined(__CYGWIN__)
-				(0.0f + 1000000));
-#else
-				1000000.0);
-#endif
+			client_max_rate / 1000000.0f);
 
 	/* Those 2 lines always stays the same */
 	fprintf(opt.f_kis_xml, "\t\t\t<carrier>IEEE 802.11b+</carrier>\n");
@@ -923,9 +930,11 @@ int dump_write_kismet_netxml(struct AP_info * ap_1st,
 
 		++network_number; // Network Number
 		strncpy(first_time, ctime(&ap_cur->tinit), TIME_STR_LENGTH - 1);
+		ENSURE(strlen(first_time) >= 1);
 		first_time[strlen(first_time) - 1] = 0; // remove new line
 
 		strncpy(last_time, ctime(&ap_cur->tlast), TIME_STR_LENGTH - 1);
+		ENSURE(strlen(last_time) >= 1);
 		last_time[strlen(last_time) - 1] = 0; // remove new line
 
 		fprintf(opt.f_kis_xml,
@@ -1187,9 +1196,11 @@ int dump_write_kismet_netxml(struct AP_info * ap_1st,
 
 			/* Write new network information */
 			strncpy(first_time, ctime(&st_cur->tinit), TIME_STR_LENGTH - 1);
+			ENSURE(strlen(first_time) >= 1);
 			first_time[strlen(first_time) - 1] = 0; // remove new line
 
 			strncpy(last_time, ctime(&st_cur->tlast), TIME_STR_LENGTH - 1);
+			ENSURE(strlen(last_time) >= 1);
 			last_time[strlen(last_time) - 1] = 0; // remove new line
 
 			fprintf(opt.f_kis_xml,
@@ -1236,12 +1247,7 @@ int dump_write_kismet_netxml(struct AP_info * ap_1st,
 								  : st_cur->rate_to;
 			fprintf(opt.f_kis_xml,
 					"\t\t<maxseenrate>%.6f</maxseenrate>\n",
-					client_max_rate /
-#if defined(__x86_64__) && defined(__CYGWIN__)
-						(0.0f + 1000000));
-#else
-						1000000.0);
-#endif
+					client_max_rate / 1000000.0f);
 
 			fprintf(opt.f_kis_xml, "\t\t<carrier>IEEE 802.11b+</carrier>\n");
 			fprintf(opt.f_kis_xml, "\t\t<encoding>CCK</encoding>\n");
