@@ -2,7 +2,7 @@
  *  802.11 ARP-request WEP packet forgery
  *  UDP, ICMP and custom packet forging developed by Martin Beck
  *
- *  Copyright (C) 2006-2020 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
+ *  Copyright (C) 2006-2022 Thomas d'Otreppe <tdotreppe@aircrack-ng.org>
  *  Copyright (C) 2004, 2005  Christophe Devine (arpforge)
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -83,7 +83,7 @@
 
 static const char usage[]
 	= "\n"
-	  "  %s - (C) 2006-2020 Thomas d\'Otreppe\n"
+	  "  %s - (C) 2006-2022 Thomas d\'Otreppe\n"
 	  "  Original work: Martin Beck\n"
 	  "  https://www.aircrack-ng.org\n"
 	  "\n"
@@ -675,12 +675,17 @@ static int read_prga_xor_ivs2(unsigned char ** dest, const char * file)
 	FILE * f;
 	int size;
 	struct ivs2_filehdr fivs2;
+	size_t file_sz = file != NULL ? strlen(file) : 0;
 
 	if (file == NULL) return (1);
-	if (*dest == NULL) *dest = (unsigned char *) malloc(1501);
+	if (*dest == NULL)
+	{
+		*dest = (unsigned char *) malloc(1501);
+		REQUIRE(*dest != NULL);
+	}
 
-	if (memcmp(file + (strlen(file) - 4), ".xor", 4) != 0
-		&& memcmp(file + (strlen(file) - 4), "." IVS2_EXTENSION, 4) != 0)
+	if (memcmp(file + (file_sz - 4), ".xor", 4) != 0
+		&& memcmp(file + (file_sz - 4), "." IVS2_EXTENSION, 4) != 0)
 	{
 		printf("Is this really a PRGA file: %s?\n", file);
 	}
@@ -953,9 +958,6 @@ int main(int argc, char * argv[])
 				break;
 
 			case ':':
-				printf("\"%s --help\" for help.\n", argv[0]);
-				return (EXIT_FAILURE);
-
 			case '?':
 				printf("\"%s --help\" for help.\n", argv[0]);
 				return (EXIT_FAILURE);
