@@ -3,7 +3,7 @@
 set -ef
 
 # Turn our pcap into a hccap
-"${top_builddir}/src/aircrack-ng${EXEEXT}" \
+"${abs_builddir}/../aircrack-ng${EXEEXT}" \
     ${AIRCRACK_NG_ARGS} \
     -J "test" \
     -e "linksys" \
@@ -11,13 +11,13 @@ set -ef
 
 # Make sure we can load it and solve it
 # NOTE: We can't load a HCCAP file. Mostly because it doesn't have a magic number
-#echo "dictionary" | "${top_builddir}/src/aircrack-ng${EXEEXT}" \
+#echo "dictionary" | "${abs_builddir}/../aircrack-ng${EXEEXT}" \
 #    ${AIRCRACK_NG_ARGS} \
 #    -e "linksys" \
 #    -w - \
 #    "test.hccap" \
 #    -l /dev/null | \
-#        grep "KEY FOUND"
+#        ${GREP} "KEY FOUND"
 
 rm -f test.hccap
 
